#include "interactive/interactive.hpp"
#include <ncurses.h>
#include <string>
#include <vector>
#include <chrono>
#include <thread>
#include <algorithm>
#include <regex>
#include "interactive.hpp"

namespace interactive {

// Helper function to convert ANSI color codes to ncurses attributes
int ansi_to_ncurses_attr(const std::string& ansi_code) {
    if (ansi_code == "0") return A_NORMAL;
    if (ansi_code == "1") return A_BOLD;
    
    // Handle colors
    if (ansi_code == "1;31") return A_BOLD | COLOR_PAIR(2);  // Red
    if (ansi_code == "1;32") return A_BOLD | COLOR_PAIR(3);  // Green
    if (ansi_code == "1;33") return A_BOLD | COLOR_PAIR(4);  // Yellow
    if (ansi_code == "1;34") return A_BOLD | COLOR_PAIR(5);  // Blue
    if (ansi_code == "1;35") return A_BOLD | COLOR_PAIR(6);  // Magenta
    if (ansi_code == "1;36") return A_BOLD | COLOR_PAIR(7);  // Cyan
    if (ansi_code == "1;37") return A_BOLD | COLOR_PAIR(8);  // White
    if (ansi_code == "90") return COLOR_PAIR(9);  // Dark Grey
    if (ansi_code == "38;5;142") return COLOR_PAIR(10); // Dark Yellow
    if (ansi_code == "38;5;250") return COLOR_PAIR(11); // Light Grey
    
    return A_NORMAL;
}

int fullscreen_window::ncurses_streambuf::overflow(int c) {
    if (c != EOF) {
        buffer += static_cast<char>(c);
        if (c == '\n') {
            // Process ANSI escape sequences
            std::string processed;
            size_t pos = 0;
            while (pos < buffer.length()) {
                if (buffer[pos] == '\033' && pos + 1 < buffer.length() && buffer[pos + 1] == '[') {
                    // Found ANSI escape sequence
                    size_t end = buffer.find('m', pos);
                    if (end != std::string::npos) {
                        std::string ansi_code = buffer.substr(pos + 2, end - pos - 2);
                        int ncurses_attr = ansi_to_ncurses_attr(ansi_code);
                        wattron(win, ncurses_attr);
                        pos = end + 1;
                    } else {
                        // Invalid ANSI sequence, just print it
                        processed += buffer[pos++];
                    }
                } else {
                    processed += buffer[pos++];
                }
            }
            
            wprintw(win, "%s", processed.c_str());
            wrefresh(win);
            buffer.clear();
        }
    }
    return c;
}

fullscreen_window::fullscreen_window(std::string title) {
    // Initialize ncurses
    initscr();
    start_color();
    use_default_colors();
    
    // Initialize color pairs
    init_pair(1, COLOR_BLACK, -1);    // Black
    init_pair(2, COLOR_RED, -1);      // Red
    init_pair(3, COLOR_GREEN, -1);    // Green
    init_pair(4, COLOR_YELLOW, -1);   // Yellow
    init_pair(5, COLOR_BLUE, -1);     // Blue
    init_pair(6, COLOR_MAGENTA, -1);  // Magenta
    init_pair(7, COLOR_CYAN, -1);     // Cyan
    init_pair(8, COLOR_WHITE, -1);    // White
    init_pair(9, COLOR_BLACK, -1);    // Dark Grey
    init_pair(10, COLOR_YELLOW, -1);  // Dark Yellow
    init_pair(11, COLOR_WHITE, -1);   // Light Grey
    
    raw();  // Use raw mode for better control
    noecho();
    keypad(stdscr, TRUE);  // Enable keypad
    curs_set(0);  // Hide cursor
    set_escdelay(25);  // Set ESC delay to 25ms
    refresh();

    // Initialize color pairs
    init_pair(12, COLOR_RED, -1);
    init_pair(13, COLOR_GREEN, -1);
    init_pair(14, COLOR_YELLOW, -1);
    init_pair(15, COLOR_BLUE, -1);
    init_pair(16, COLOR_MAGENTA, -1);
    init_pair(17, COLOR_CYAN, -1);
    init_pair(18, COLOR_WHITE, -1);
    
    // Create display window (takes up all but bottom 4 lines)
    int max_y, max_x;
    getmaxyx(stdscr, max_y, max_x);
    display_win = newwin(max_y - 4, max_x, 0, 0);
    scrollok(display_win, TRUE);
    keypad(display_win, TRUE);  // Enable keypad for display window
    wrefresh(display_win);

    // Create input window (bottom 4 lines)
    input_win = newwin(4, max_x, max_y - 4, 0);
    box(input_win, 0, 0);
    keypad(input_win, TRUE);  // Enable keypad for input window
    wrefresh(input_win);

    // Set up output redirection
    cout_buf = new ncurses_streambuf(display_win);
    cerr_buf = new ncurses_streambuf(display_win);
    
    // Save original streambufs
    original_cout = std::cout.rdbuf();
    original_cerr = std::cerr.rdbuf();
    
    // Redirect cout and cerr to our custom streambuf
    std::cout.rdbuf(cout_buf);
    std::cerr.rdbuf(cerr_buf);
}

void fullscreen_window::close() {
    if (is_closed) return;
    is_closed = true;

    // Restore original streambufs
    std::cout.rdbuf(original_cout);
    std::cerr.rdbuf(original_cerr);
    
    // Clean up streambufs
    delete cout_buf;
    delete cerr_buf;
    
    // Clean up ncurses windows
    delwin(display_win);
    delwin(input_win);
    
    // End ncurses
    endwin();
}

fullscreen_window::~fullscreen_window() {
    close();
}

void fullscreen_window::set_input_text_display(std::string text) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Center the text
    int max_y, max_x;
    getmaxyx(input_win, max_y, max_x);
    int start_x = (max_x - text.length()) / 2;
    mvwprintw(input_win, max_y/2, start_x, "%s", text.c_str());
    
    wrefresh(input_win);
}

std::string fullscreen_window::set_input_text_entry(std::string prompt) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    wrefresh(input_win);
    
    // Set up input
    echo();
    curs_set(1);
    std::string input;
    int ch;
    int x = prompt.length() + 3;
    
    while ((ch = wgetch(input_win)) != '\n') {
        if (ch == 27) { // ESC key
            input = "";
            break;
        }
        if (ch == KEY_BACKSPACE || ch == 127) {
            if (!input.empty()) {
                input.pop_back();
                x--;
                mvwdelch(input_win, 1, x);
            }
        } else if (isprint(ch)) {
            input += ch;
            mvwaddch(input_win, 1, x++, ch);
        }
    }
    
    noecho();
    curs_set(0);
    return input;
}

std::string fullscreen_window::set_input_multiple_choice(std::string prompt, std::vector<std::string> choices) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    
    int selected = 0;
    std::string filter = "";
    std::string last_filter = "";
    auto last_key_time = std::chrono::steady_clock::now();
    int scroll_offset = 0;
    int last_selected = -1;
    int last_scroll_offset = -1;
    
    // Pre-compute lowercase versions of choices for case-insensitive matching
    std::vector<std::string> lowercase_choices;
    lowercase_choices.reserve(choices.size());
    for (const auto& choice : choices) {
        std::string lower;
        lower.reserve(choice.size());
        std::transform(choice.begin(), choice.end(), std::back_inserter(lower), ::tolower);
        lowercase_choices.push_back(std::move(lower));
    }
    
    // Initial filtered choices (all choices)
    std::vector<std::string> filtered_choices = choices;
    
    while (true) {
        // Only recalculate filtered choices if filter changed
        if (filter != last_filter) {
            if (filter.empty()) {
                filtered_choices = choices;
            } else {
                filtered_choices.clear();
                // Convert filter to lowercase once
                std::string lowercase_filter;
                lowercase_filter.reserve(filter.size());
                std::transform(filter.begin(), filter.end(), std::back_inserter(lowercase_filter), ::tolower);
                
                // Filter using pre-computed lowercase versions
                for (size_t i = 0; i < choices.size(); ++i) {
                    if (lowercase_choices[i].find(lowercase_filter) == 0) {
                        filtered_choices.push_back(choices[i]);
                    }
                }
            }
        }
        
        // Get window dimensions
        int max_y, max_x;
        getmaxyx(input_win, max_y, max_x);
        int available_width = max_x - 4; // -4 for borders
        
        // Calculate scroll offset to keep selected item visible
        if (selected >= 0 && selected < filtered_choices.size()) {
            int item_start = 0;
            for (int i = 0; i < selected; i++) {
                item_start += filtered_choices[i].length() + 3;
            }
            int item_end = item_start + filtered_choices[selected].length();
            
            if (item_start < scroll_offset) {
                scroll_offset = item_start;
            } else if (item_end > scroll_offset + available_width) {
                scroll_offset = item_end - available_width;
            }
        }
        
        // Only redraw if selection or scroll changed
        if (selected != last_selected || scroll_offset != last_scroll_offset || last_filter != filter) {
            // Clear the input window except for the prompt
            werase(input_win);
            box(input_win, 0, 0);
            mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
            
            // Display filtered choices horizontally
            int x = 2 - scroll_offset;
            for (size_t i = 0; i < filtered_choices.size(); i++) {
                if (i == selected) {
                    wattron(input_win, A_REVERSE);
                }
                
                // Only draw if visible
                if (x + filtered_choices[i].length() > 0 && x < max_x - 2) {
                    mvwprintw(input_win, 2, x, "%s", filtered_choices[i].c_str());
                }
                
                if (i == selected) {
                    wattroff(input_win, A_REVERSE);
                }
                
                x += filtered_choices[i].length() + 3; // +3 for " | " separator
                
                // Draw separator if not last item
                if (i < filtered_choices.size() - 1) {
                    if (x - 3 + 3 > 0 && x - 3 < max_x - 2) {
                        mvwprintw(input_win, 2, x - 3, " | ");
                    }
                }
            }
            
            wrefresh(input_win);
            
            // Update last state
            last_selected = selected;
            last_scroll_offset = scroll_offset;
            last_filter = filter;
        }
        
        int ch = wgetch(input_win);
        
        // Handle key input
        switch (ch) {
            case '\n':
                if (!filtered_choices.empty()) {
                    return filtered_choices[selected];
                }
                break;
                
            case 27: // ESC key
                if (!filter.empty()) {
                    // Clear the filter and reset selection
                    filter = "";
                    selected = 0;
                    scroll_offset = 0;
                } else {
                    // Only return if filter is already empty
                    return "";
                }
                break;
                
            case KEY_LEFT:
            case KEY_UP:
                if (selected > 0) {
                    selected--;
                }
                break;
                
            case KEY_RIGHT:
            case KEY_DOWN:
                if (selected < filtered_choices.size() - 1) {
                    selected++;
                }
                break;
                
            default:
                if (isalnum(ch)) {
                    auto now = std::chrono::steady_clock::now();
                    auto elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(now - last_key_time).count();
                    
                    if (elapsed > 500) { // Reset filter if too much time has passed
                        filter = "";
                    }
                    
                    filter += ch;
                    selected = 0;
                    last_key_time = now;
                }
                break;
        }
    }
}

bool fullscreen_window::set_input_yes_no(std::string prompt) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    
    int selected = 0; // 0 for Yes, 1 for No
    const char* options[] = {"Yes", "No"};
    
    while (true) {
        // Display options
        for (int i = 0; i < 2; i++) {
            if (i == selected) {
                wattron(input_win, A_REVERSE);
            }
            mvwprintw(input_win, 2, 2 + i * 10, "%s", options[i]);
            if (i == selected) {
                wattroff(input_win, A_REVERSE);
            }
        }
        
        wrefresh(input_win);
        
        int ch = wgetch(input_win);
        
        if (ch == '\n') {
            return selected == 0;
        } else if (ch == 'y' || ch == 'Y') {
            return true;
        } else if (ch == 'n' || ch == 'N') {
            return false;
        } else if (ch == KEY_LEFT) {
            if (selected > 0) {
                selected = 0;
            }
        } else if (ch == KEY_RIGHT) {
            if (selected < 1) {
                selected = 1;
            }
        }
    }
}

char fullscreen_window::set_input_yes_no_cancel(std::string prompt) {
    werase(input_win);
    box(input_win, 0, 0);
    
    // Display prompt
    mvwprintw(input_win, 1, 2, "%s", prompt.c_str());
    
    int selected = 0; // 0 for Yes, 1 for No, 2 for Cancel
    const char* options[] = {"Yes", "No", "Cancel"};
    
    while (true) {
        // Display options
        for (int i = 0; i < 3; i++) {
            if (i == selected) {
                wattron(input_win, A_REVERSE);
            }
            mvwprintw(input_win, 2, 2 + i * 10, "%s", options[i]);
            if (i == selected) {
                wattroff(input_win, A_REVERSE);
            }
        }
        
        wrefresh(input_win);
        
        int ch = wgetch(input_win);
        
        if (ch == '\n') {
            return selected == 0 ? 'y' : (selected == 1 ? 'n' : 'c');
        } else if (ch == 'y' || ch == 'Y') {
            return 'y';
        } else if (ch == 'n' || ch == 'N') {
            return 'n';
        } else if (ch == 'c' || ch == 'C') {
            return 'c';
        } else if (ch == KEY_LEFT) {
            if (selected > 0) {
                selected--;
            }
        } else if (ch == KEY_RIGHT) {
            if (selected < 2) {
                selected++;
            }
        }
    }
}

} // namespace interactive